import * as wasm from './wasm_unic_bg';

let cachedTextDecoder = new TextDecoder('utf-8');

let cachegetUint8Memory = null;
function getUint8Memory() {
    if (cachegetUint8Memory === null || cachegetUint8Memory.buffer !== wasm.memory.buffer) {
        cachegetUint8Memory = new Uint8Array(wasm.memory.buffer);
    }
    return cachegetUint8Memory;
}

function getStringFromWasm(ptr, len) {
    return cachedTextDecoder.decode(getUint8Memory().subarray(ptr, ptr + len));
}

let cachedGlobalArgumentPtr = null;
function globalArgumentPtr() {
    if (cachedGlobalArgumentPtr === null) {
        cachedGlobalArgumentPtr = wasm.__wbindgen_global_argument_ptr();
    }
    return cachedGlobalArgumentPtr;
}

let cachegetUint32Memory = null;
function getUint32Memory() {
    if (cachegetUint32Memory === null || cachegetUint32Memory.buffer !== wasm.memory.buffer) {
        cachegetUint32Memory = new Uint32Array(wasm.memory.buffer);
    }
    return cachegetUint32Memory;
}
/**
* @returns {Version}
*/
export function get_unicode_version() {
    return Version.__wrap(wasm.get_unicode_version());
}

let WASM_VECTOR_LEN = 0;

let cachedTextEncoder = new TextEncoder('utf-8');

let passStringToWasm;
if (typeof cachedTextEncoder.encodeInto === 'function') {
    passStringToWasm = function(arg) {


        let size = arg.length;
        let ptr = wasm.__wbindgen_malloc(size);
        let offset = 0;
        {
            const mem = getUint8Memory();
            for (; offset < arg.length; offset++) {
                const code = arg.charCodeAt(offset);
                if (code > 0x7F) break;
                mem[ptr + offset] = code;
            }
        }

        if (offset !== arg.length) {
            arg = arg.slice(offset);
            ptr = wasm.__wbindgen_realloc(ptr, size, size = offset + arg.length * 3);
            const view = getUint8Memory().subarray(ptr + offset, ptr + size);
            const ret = cachedTextEncoder.encodeInto(arg, view);

            offset += ret.written;
        }
        WASM_VECTOR_LEN = offset;
        return ptr;
    };
} else {
    passStringToWasm = function(arg) {


        let size = arg.length;
        let ptr = wasm.__wbindgen_malloc(size);
        let offset = 0;
        {
            const mem = getUint8Memory();
            for (; offset < arg.length; offset++) {
                const code = arg.charCodeAt(offset);
                if (code > 0x7F) break;
                mem[ptr + offset] = code;
            }
        }

        if (offset !== arg.length) {
            const buf = cachedTextEncoder.encode(arg.slice(offset));
            ptr = wasm.__wbindgen_realloc(ptr, size, size = offset + buf.length);
            getUint8Memory().set(buf, ptr + offset);
            offset += buf.length;
        }
        WASM_VECTOR_LEN = offset;
        return ptr;
    };
}

let cachegetInt32Memory = null;
function getInt32Memory() {
    if (cachegetInt32Memory === null || cachegetInt32Memory.buffer !== wasm.memory.buffer) {
        cachegetInt32Memory = new Int32Array(wasm.memory.buffer);
    }
    return cachegetInt32Memory;
}

function getArrayI32FromWasm(ptr, len) {
    return getInt32Memory().subarray(ptr / 4, ptr / 4 + len);
}
/**
*
* Get segmented Unicode scalars of a string with each segmentation representing a Unicode
* grapheme cluster.
*
* Returns a list of 32-bit signed integers where:
* - Integers equal or greater than 0 represent a code point of a Unicode character
* - Integers less than 0 are separators of the segmentations
* @param {string} string
* @returns {Int32Array}
*/
export function get_segmented_scalars(string) {
    const ptr0 = passStringToWasm(string);
    const len0 = WASM_VECTOR_LEN;
    const retptr = globalArgumentPtr();
    try {
        wasm.get_segmented_scalars(retptr, ptr0, len0);
        const mem = getUint32Memory();
        const rustptr = mem[retptr / 4];
        const rustlen = mem[retptr / 4 + 1];

        const realRet = getArrayI32FromWasm(rustptr, rustlen).slice();
        wasm.__wbindgen_free(rustptr, rustlen * 4);
        return realRet;


    } finally {
        wasm.__wbindgen_free(ptr0, len0 * 1);

    }

}

/**
*
* Convert a Unicode code point to a character.
* If the code point is invalid, U+FFFD is returned.
* @param {number} code_point
* @returns {string}
*/
export function get_character_from_code_point(code_point) {
    return String.fromCodePoint(wasm.get_character_from_code_point(code_point));
}

/**
* @param {string} chr
* @returns {number}
*/
export function get_code_point(chr) {
    return wasm.get_code_point(chr.codePointAt(0)) >>> 0;
}

/**
* @param {string} chr
* @returns {string}
*/
export function get_name(chr) {
    const retptr = globalArgumentPtr();
    wasm.get_name(retptr, chr.codePointAt(0));
    const mem = getUint32Memory();
    const rustptr = mem[retptr / 4];
    const rustlen = mem[retptr / 4 + 1];

    const realRet = getStringFromWasm(rustptr, rustlen).slice();
    wasm.__wbindgen_free(rustptr, rustlen * 1);
    return realRet;

}

/**
* @param {string} chr
* @returns {Version}
*/
export function get_age(chr) {
    return Version.__wrap(wasm.get_age(chr.codePointAt(0)));
}

/**
* @param {string} chr
* @returns {GeneralCategory}
*/
export function get_general_category(chr) {
    return GeneralCategory.__wrap(wasm.get_general_category(chr.codePointAt(0)));
}

/**
*/
export class GeneralCategory {

    static __wrap(ptr) {
        const obj = Object.create(GeneralCategory.prototype);
        obj.ptr = ptr;

        return obj;
    }

    free() {
        const ptr = this.ptr;
        this.ptr = 0;

        wasm.__wbg_generalcategory_free(ptr);
    }
    /**
    * @returns {string}
    */
    get_human_readable_name() {
        const retptr = globalArgumentPtr();
        wasm.generalcategory_get_human_readable_name(retptr, this.ptr);
        const mem = getUint32Memory();
        const rustptr = mem[retptr / 4];
        const rustlen = mem[retptr / 4 + 1];

        const realRet = getStringFromWasm(rustptr, rustlen).slice();
        wasm.__wbindgen_free(rustptr, rustlen * 1);
        return realRet;

    }
    /**
    * @returns {string}
    */
    get_abbreviation() {
        const retptr = globalArgumentPtr();
        wasm.generalcategory_get_abbreviation(retptr, this.ptr);
        const mem = getUint32Memory();
        const rustptr = mem[retptr / 4];
        const rustlen = mem[retptr / 4 + 1];

        const realRet = getStringFromWasm(rustptr, rustlen).slice();
        wasm.__wbindgen_free(rustptr, rustlen * 1);
        return realRet;

    }
}
/**
*/
export class Version {

    static __wrap(ptr) {
        const obj = Object.create(Version.prototype);
        obj.ptr = ptr;

        return obj;
    }

    free() {
        const ptr = this.ptr;
        this.ptr = 0;

        wasm.__wbg_version_free(ptr);
    }
    /**
    * @returns {number}
    */
    get major() {
        return wasm.__wbg_get_version_major(this.ptr);
    }
    /**
    * @param {number} arg0
    * @returns {void}
    */
    set major(arg0) {
        return wasm.__wbg_set_version_major(this.ptr, arg0);
    }
    /**
    * @returns {number}
    */
    get minor() {
        return wasm.__wbg_get_version_minor(this.ptr);
    }
    /**
    * @param {number} arg0
    * @returns {void}
    */
    set minor(arg0) {
        return wasm.__wbg_set_version_minor(this.ptr, arg0);
    }
    /**
    * @returns {number}
    */
    get micro() {
        return wasm.__wbg_get_version_micro(this.ptr);
    }
    /**
    * @param {number} arg0
    * @returns {void}
    */
    set micro(arg0) {
        return wasm.__wbg_set_version_micro(this.ptr, arg0);
    }
}

export const __wbindgen_throw = function(arg0, arg1) {
    let varg0 = getStringFromWasm(arg0, arg1);
    throw new Error(varg0);
};

